require("inventory")
local json = require("json")

local InventoryInfo = {
    setkey = "key200002",
    savedSets = {},
    savedSections = {},

    displayErrorMessage = nil,

    initialize = function(self, back)
        self.savedSets = {}
        self.savedSections = {}
        osrs.Persistence:getValue(self.setkey,
        function(_, callbackValue)
            if(callbackValue == nil) then
                self.displayErrorMessage("Nothing saved.")
                back()
                return
            end
            local result = json.decode(callbackValue)
            if(type(result) == "string") then
                result = {result}
            end
            for index, section in pairs(result["sectionNames"]) do
                table.insert(self.savedSections, section)
            end
            local taskList = AsyncTaskList.New() 
            for index, name in pairs(result["setNames"]) do
                AsyncTaskList.schedule(taskList,function(callback) self:setupSet(name, index, callback) end)
            end
            AsyncTaskList.schedule(taskList,function(callback) back() end)
            AsyncTaskList.runTaskList(taskList)
        end
        )
    end,

    setupSet = function(self, name, index, callback)
        osrs.Persistence:getValue(name, 
        function(key, callbackValue)
            if(callbackValue == nil) then
                self.displayErrorMessage("No loadout of that name.")
                callback()
            else
                inv = Inventory:remake(nil, name, json.decode(callbackValue))
                inv.index = index
                table.insert(self.savedSets, inv)
                callback()
            end
        end
        )
    end,

    importSetup = function(self, info, callback)
        inv = {}
        if pcall(function() info = json.decode(info) end) and pcall(function() inv = Inventory:remake(nil, info["name"], info) end) then
            if self:checkName(info["name"], self.savedSets) == 0 then
                inv.index = index
                table.insert(self.savedSets, inv)
            else
                self.displayErrorMessage("Given import matches an already made Setup.")
            end
        else
            self.displayErrorMessage("Given import string does not match the layout that is requiered.")
        end
        callback()
    end,

    importSection = function(self, info, callback)
        if pcall(function() info = json.decode(info) end) and info["name"] ~= nil and info["setups"] ~= nil then
            if self:checkName(info["name"], self.savedSections) == 0 then
                table.insert(self.savedSections, info["name"])
                for index, setup in pairs(info["setups"]) do
                    for i, s in pairs(self.savedSets) do
                        if s.name == setup then
                            table.insert(s.sections, info["name"])
                        end
                    end
                end
            else
                self.displayErrorMessage("Given import matches an already made Section.")
            end
        else
            self.displayErrorMessage("Given import string does not match the layout that is requiered.")
        end
        callback()
    end,

    checkName = function(self, name, list)
        if(name == "") then
            self.displayErrorMessage("No name provided.")
            return -1
        elseif (type(name) ~= "string") then
            self.displayErrorMessage(string.format("Invalid type passed. Value: %s, Type: %s", name, type(name)))
            return -1
        end
        for index, set in pairs(list) do
            if name == set or name == set.name then 
                self.displayErrorMessage("That name is already in use!")
                return -1
            end
        end
        return 0
    end, 

    createSection = function(self, section, callback)
        if self:checkName(section, self.savedSections) == 0 then
            table.insert(self.savedSections, section)
        end
        callback()
    end,

    createInventory = function(self, inv, callback)
        if self:checkName(inv.name, self.savedSets) == 0 then
            inv.index = #self.savedSets + 1
            table.insert(self.savedSets, inv)
        end
        callback()
    end,

    saveSets = function(self)
        save = {}
        save["setNames"] = {}
        save["sectionNames"] = {}
        for index, set in pairs(self.savedSets) do
            if set ~= nil then 
                table.insert(save["setNames"], set.name)
                self:saveInventory(set)
            end
        end
        for index, section in pairs(self.savedSections) do
            if section ~= nil then
                table.insert(save["sectionNames"], section)
            end
        end
        osrs.Persistence:setValue(self.setkey, json.encode(save),
        function(key, success)
            if(success) then
                osrs.print("Set Key Saved.") 
            else
                osrs.print("Set Key failed.")
            end
        end)
    end,

    writeInventory = function(self, inv)
        local info = {}
        info["name"] = inv.name
        info["items"] = {}
        info["other"] = {}
        info["sections"] = {}
        table.insert(info["other"], inv.favorite)
        table.insert(info["other"], inv.ordered)
        table.insert(info["other"], inv.highlighting)
        table.insert(info["other"], inv.notes)
        for index, section in pairs(inv.sections) do
            table.insert(info["sections"], section)
        end
        for index, item in pairs(inv.invItems) do
            table.insert(info["items"], item:getItem())
        end
        for index, item in pairs(inv.runePouch) do
            table.insert(info["items"], item:getItem())
        end
        for index, item in pairs(inv.equipment) do
            table.insert(info["items"], item:getItem())
        end
        table.insert(info["items"], inv.spellbook:getItem())
        return json.encode(info)
    end,

    writeSection = function(self, section)
        local info = {}
        info["name"] = section 
        info["setups"] = {}
        for index, setup in pairs(self.savedSets) do
            for i, sect in pairs (setup.sections) do
                if sect == section then 
                    table.insert(info["setups"], setup.name)
                end
            end
        end
        return json.encode(info)
    end,

    saveInventory = function(self, inv)
        osrs.Persistence:setValue(inv.name, self:writeInventory(inv),
        function(key, success)
            if(success) then
                osrs.print(string.format("Set %s Saved", inv.name)) 
            else
                osrs.print(string.format("Set %s Failed", inv.name)) 
            end
        end)
    end,

    resetInventory = function(self, temp)
        for index, set in pairs(self.savedSets) do
            if set.name == temp.name then
                self.savedSets[index] = temp
            end
        end
    end,

    deleteSection = function(self, name)
        for index, section in pairs(self.savedSections) do
            if section == name then
                table.remove(self.savedSections, index)
                for index, set in pairs(self.savedSets) do
                    for i, section in pairs(set.sections) do
                        if section == name then
                            table.remove(set.sections, i)
                        end
                    end
                end
                return
            end
        end
        self.displayErrorMessage(string.format("No section named %s found", name))
    end,

    deleteInventory = function(self, name)
        for index, set in pairs(self.savedSets) do
            if set.name == name then 
                table.remove(self.savedSets, index)
                osrs.Persistence:setValue(name, nil, nil)
                self:deleteInventoryHelper()
                return
            end
        end
        self.displayErrorMessage(string.format("No set named %s found", name))
    end,

    deleteInventoryHelper = function(self)
        for index, set in pairs(self.savedSets) do
            set.index = index
        end
    end,

    -- clean up later
    moveItem = function(self, item, oldIndex, newIndex, list, callback)
        newList = {}
        if tonumber(newIndex) == nil then
            self.displayErrorMessage(string.format("Given %s when a number is needed", tostring(newIndex)))
            return
        end
        newIndex = tonumber(newIndex)
        if newIndex == oldIndex then 
            self.displayErrorMessage("Trying to move set its current position")
            return
        end
        if newIndex < 1 or newIndex > #list then
            self.displayErrorMessage(string.format("Set position: %s is out of bounds", tostring(newIndex)))
            return
        end
        change = newIndex - oldIndex
        if newIndex < oldIndex then 
            change = -1 * change
        end
        for index, x in pairs(list) do
            if index == newIndex then
                if item.index ~= nil then
                    item.index = index
                end
                table.insert(newList, item)
            elseif index > newIndex and newIndex < oldIndex and change ~= 0 then 
                if item.index ~= nil then
                    list[index - 1].index = index
                end
                table.insert(newList, list[index - 1])
                change = change - 1
            elseif index < newIndex and index >= oldIndex and newIndex > oldIndex and change ~= 0 then 
                if item.index ~= nil then
                    list[index + 1].index = index
                end
                table.insert(newList, list[index + 1])
                change = change - 1
            else
                if item.index ~= nil then
                    x.index = index
                end
                table.insert(newList, x)
            end
        end
        if item.index ~= nil then
            self.savedSets = newList
        else
            self.savedSections = newList
        end
        callback()
    end,

    changeItem = function(self, item, itemIndex, group)
        itemChanged = 0
        for index, set in pairs(self.savedSets) do
            itemChanged = set:changeItem(item, itemIndex, group)
        end
        return itemChanged
    end,

    renameSection = function(self, oldName, newName)
        if self:checkName(newName, self.savedSections) ~= -1 then
            for index, section in pairs(self.savedSections) do
                if oldName == section then
                    self.savedSections[index] = newName
                end
            end
            for index, set in pairs(self.savedSets) do
                for i, section in pairs(set.sections) do
                    if section == oldName then
                        set.sections[i] = newName
                    end
                end
            end
        end
    end,

    renameSet = function(self, oldName, newName)
        if self:checkName(newName, self.savedSets) ~= -1 then 
            for index, set in pairs(self.savedSets) do
                if set.name == oldName then
                    self.savedSets[index].name = newName
                    osrs.Persistence:setValue(oldName, nil, nil)
                end
            end
        end
    end
}

return InventoryInfo