local ThemeTool = require("UI/themeTool")

local popupMenu = {
    window = nil,
    panel = nil,
    cancelButton = nil, 

    isOpen = nil,

    onCreate = nil,
    onImport = nil,
    onDelete = nil,
    onRefresh = nil,
    onRename = nil,

    initialize = function(self)
        self.window =  osrs.Ui:window(
            {
                collapsable = false,
                sizeMode = 1,
                width = 350,
                height = 200,
                x = 400,
            }
        )
        self.panel = osrs.Ui:pane(
            {
                border = false,
            }
        )
        self.cancelButton = osrs.Ui:textButton(
            {
                text = "Cancel",
                onClick = function()
                    self:closeWindow()
                end
            }
        )
    end,

    closeWindow = function(self)
        if self.isOpen then 
            osrs.Ui.canvas:removeChild(self.window)
        end
        self.isOpen = false
    end,

    resetPanel = function(self)
        self.window:removeChild(self.panel)
        self.panel = osrs.Ui:pane(
            {
                border = false,
            }
        )
    end,

    createPopup = function(self)
        self.window.title = "Create"
        local setupButton = osrs.Ui:textButton(
            {
                text = "Add new setup...",
                onClick = function()
                    self:conformationPopup("Add New Setup", 
                    "Enter the name of this setup:",
                    true,
                    "",
                    function() 
                        self.onCreate(Inventory:new(nil, nametextbox.value), true)
                    end)
                end
            }
        )
        local sectionButton = osrs.Ui:textButton(
            {
                text = "Add new section...",
                onClick = function()
                    self:conformationPopup("Add New Section", 
                    "Enter the name of this section:",
                    true,
                    "",
                    function() 
                        self.onCreate(nametextbox.value, false) 
                    end)
                end
            }
        )
        self:showWindow({setupButton, sectionButton})
    end,

    importPopup = function(self)
        self.window.title = "Import"
        local setupButton = osrs.Ui:textButton(
            {
                text = "Import setup...",
                onClick = function()
                    self:conformationPopup("Import New Setup", 
                    "Enter Setup Data:",
                    true,
                    "",
                    function() 
                        self.onImport(nametextbox.value, true) 
                    end)
                end
            }
        )
        local sectionButton = osrs.Ui:textButton(
            {
                text = "Import section...",
                onClick = function()
                    self:conformationPopup("Add New Section", 
                    "Enter Section Data:",
                    true,
                    "",
                    function() 
                        self.onImport(nametextbox.value, false) 
                    end)
                end
            }
        )
        self:showWindow({setupButton, sectionButton})
    end,

    deletePopup = function(self, group, name, isSet)
        self:conformationPopup(
            string.format("Delete %s", group), 
            "Are you sure?",
            false,
            "",
            function() 
                self.onDelete(name, isSet) 
            end)
    end,

    holderPopup = function(self, item, index, total, itemType, currentSection, list, sectionMode)
        self.window.title = itemType
        local renameButton = osrs.Ui:textButton(
            {
                text = string.format("Rename %s", itemType),
                onClick = function()
                    if itemType == "Section" then 
                        self:renamePopupHelper(item, false)
                    else
                        self:renamePopupHelper(item.name, true)
                    end
                end
            }
        )
        local moveUpButton = osrs.Ui:textButton(
            {
                text = string.format("Move %s Up", itemType),
                onClick = function()
                    self:closeWindow()
                    self.onMove(item, index, index - 1)
                end
            }
        )
        local moveDownButton = osrs.Ui:textButton(
            {
                text = string.format("Move %s Down", itemType),
                onClick = function()
                    self:closeWindow()
                    self.onMove(item, index, index + 1)
                end
            }
        )
        local moveTopButton = osrs.Ui:textButton(
            {
                text = string.format("Move %s To Top", itemType),
                onClick = function()
                    self:closeWindow()
                    self.onMove(item, index, 1)
                end
            }
        )
        local moveBottomButton = osrs.Ui:textButton(
            {
                text = string.format("Move %s To Bottom", itemType),
                onClick = function()
                    self:closeWindow()
                    self.onMove(item, index, total)
                end
            }
        )
        local movePositionButton = osrs.Ui:textButton(
            {
                text = string.format("Move %s To Position...", itemType),
                onClick = function()
                    self:closeWindow()
                    self:conformationPopup(string.format("Move %s", itemType),
                    string.format("Enter a position between 1 and %s. Current position is %s.",
                    tostring(total), tostring(index)),
                    true,
                    "",
                    function()
                    self.onMove(item, index, nametextbox.value)
                    end)
                end
            }
        )
        ui = {renameButton, moveUpButton, moveDownButton, moveTopButton, moveBottomButton, movePositionButton}
        if itemType == "Section" then
            ui = self:sectionButtons(ui, item, list)
        elseif sectionMode then
            ui = self:setButtons(ui, item, currentSection, list)
        end
        self:showWindow(ui)
    end,

    
    sectionButtons = function(self, ui, item, list)
        local sectionAddButton = osrs.Ui:textButton(
            {
                text = "Add setup to sections...",
                onClick = function()
                    self:showToggles("Select Inventory Setups", 
                    function(given) self:sectionHelper(given, list, item) end, 
                    list)
                end
            }
        )
        local deleteButton = osrs.Ui:textButton(
            {
                text = "Delete Section...",
                onClick = function()
                    self:deletePopup("Section", item, false)
                end
            }
        )
        table.insert(ui, sectionAddButton)
        table.insert(ui, deleteButton)
        return ui
    end,

    sectionHelper = function(self, given, list, item)
        for index, setName in pairs(given) do
            for i, set in pairs(list) do
                if set.name == setName then 
                    found = 0 
                    for _, section in pairs(set.sections) do 
                        if section == item then
                            found = 1
                        end
                    end
                    if found == 0 then
                        table.insert(set.sections, item)
                    end
                end
            end
        end
    end,

    setButtons = function(self, ui, item, currentSection, list)
        -- removes the unassigned section from the list
        table.remove(list, #list)
        sectionAddButton = osrs.Ui:textButton(
            {
                text = "Add setup to sections...",
                onClick = function()
                    self:showToggles("Select Sections",
                    function(list) item:addSections(list) end, 
                    list)
                end
            }
        )
        sectionRemoveButton = osrs.Ui:textButton(
            {
                text = "Remove setup from section",
                onClick = function()
                    self:closeWindow()
                    table.remove(item.sections, currentSection)
                    self.onRefresh()
                end
            }
        )
        table.insert(ui, sectionAddButton)
        if currentSection ~= nil then
            table.insert(ui, sectionRemoveButton)
        end
        return ui
    end,

    makeToggle = function(self, ui, name, val, tracker)
        local toggle = osrs.Ui:checkbox(
            {
                label = name,
                value = val,
                changed = function(value)
                    val = value
                    if val then
                        table.insert(tracker, name)
                    else
                        for i, object in pairs(tracker) do 
                            if object == name then 
                                table.remove(tracker, i)
                            end
                        end
                    end
                end
            }
        )
        table.insert(ui, toggle)
    end,

    showToggles = function(self, title, onClick, list)
        addedTo = {}
        ui = {}
        self.window.title = title
        for index, i in pairs(list) do
            i.on = false
            if i.text ~= nil then
                self:makeToggle(ui, i.text, i.on, addedTo)
            else
                self:makeToggle(ui, i.name, i.on, addedTo)
            end
        end
        local submitButton = osrs.Ui:textButton(
            {
                text = "Submit",
                onClick = function()
                    self:closeWindow()
                    onClick(addedTo)
                    self.onRefresh()
                end
            }
        )
        table.insert(ui, submitButton)
        self:showWindow(ui)
    end,


    renamePopupHelper = function(self, name, isSet)
        self:conformationPopup("New Name", 
        "Enter a New Name:",
        true,
        "",
        function() 
            self.onRename(name, nametextbox.value, isSet)
        end)
    end,

    conformationPopup = function(self, title, text, addtextbox, textboxValue, clickOK)
        self.window.title = title
        local prompt = osrs.Ui:textbox(
            {
                text = text,
            }
        )
        local okButton = osrs.Ui:textButton(
            {
                text = "Ok",
                onClick = function()
                    self:closeWindow()
                    clickOK()
                end
            }
        )
        if addtextbox then
            nametextbox = osrs.Ui:inputText(
                {
                    sizeMode = 1,
                    maxSize = 5000,
                    sizeMode = 1,
                    width = 200,
                    height = 30,
                }
            )
            if textboxValue ~= nil then 
                nametextbox.value = textboxValue
            end
            self:showWindow({prompt, nametextbox, okButton})
        else
            self:showWindow({prompt, okButton})
        end
    end,

    errorPopup = function(self, message)
        self.window.title = "ERROR"
        local prompt = osrs.Ui:textbox(
            {
                text = string.format("ERROR: %s", message),
            }
        )
        self:showWindow({prompt})
    end,

    showWindow = function(self, items)
        self:resetPanel()
        for index, item in pairs(items) do
            self.panel:insertChild(index, item)
        end
        self.panel:addChild(self.cancelButton)
        self.window:addChild(self.panel)
        ThemeTool:update(self.window)
        osrs.Ui.canvas:addChild(self.window)
        self.isOpen = true
    end,
}

return popupMenu