local InventoryPanel = require("UI/inventoryPanel")
local RunePanel = require("UI/runePanel")
local EquipmentPanel = require("UI/equipmentPanel")
local SpellbookPanel = require("UI/spellbookPanel")
local ThemeTool = require("UI/themeTool")

-- UI representation of a set --
local InventorySetWindow = {
    onClose = nil,
    onRefresh = nil,
    onChange = nil,

    inventory = nil,
    highlightingID = nil,
    window = nil,
    panel = nil,
    resetButton = nil,
    invPanel = nil,
    runePanel = nil,
    equipPanel = nil,
    spellbookPanel = nil,
    notesTitle = nil,
    notesText = nil,
    closeButton = nil,
    deleteButton = nil,
    popupMenu = nil,

    initialize = function(self, inventory)
        local taskList = AsyncTaskList.New() 
        AsyncTaskList.schedule(taskList,function(callback) self:create(inventory) callback() end)  
        AsyncTaskList.schedule(taskList,function(callback) self:show() callback() end)
        AsyncTaskList.schedule(taskList,function(callback) self:toggleHighliting(true) callback() end)
        AsyncTaskList.runTaskList(taskList)
    end,

    create = function(self, inventory)
        self.inventory = inventory
        self.window = osrs.Ui:window(
            {
                title = self.inventory.name,
                collapsable = false
            }
        )
        self.window.sizeMode = 1
        self.window.width = 300
        self.window.height = 800
        self.window.positionMode = 1
        self.panel = osrs.Ui:pane(
            {
                border = false
            }
        )
        self.resetButton = osrs.Ui:textButton(
            {
                text = "Refresh",
                sameLine = true,
                onClick = function()
                    self.popupMenu:conformationPopup("Warning", 
                    "Are you sure you want to update this inventory setup?",
                    false,
                    "",
                    function()
                        self.onRefresh(Inventory:new(nil, inventory.name))
                    end)
                end
            }
        )
        self.invPanel = InventoryPanel
        self.invPanel:create(self.inventory.invItems, self.inventory.highlighting)
        self.invPanel.onClick = function(item, index, table) self:onClick(item, index, table) end
        self.runePanel = RunePanel
        self.runePanel:create(self.inventory.runePouch, self.inventory.highlighting)
        self.runePanel.onClick = function(item, index, table) self:onClick(item, index, table) end
        self.equipPanel = EquipmentPanel
        self.equipPanel:create(self.inventory.equipment, self.inventory.highlighting)
        self.equipPanel.onClick = function(item, index, table) self:onClick(item, index, table) end
        self.spellbookPanel = SpellbookPanel
        self.spellbookPanel.popupMenu = self.popupMenu
        self.spellbookPanel:create(self.inventory.spellbook, self.inventory.highlighting)
        self.spellbookPanel.onSwap = function(book) self:onSwap(book) end
        self.notesTitle = osrs.Ui:textbox(
            {
                text = "Notes",
            }
        )
        self.notesText = osrs.Ui:inputText(
            {
                value = self.inventory.notes,
                sizeMode = 1,
                maxSize = 500,
                height = 200,
                width = 300
            }
        )
        self.closeButton = osrs.Ui:textButton(
            {
                text = "Close",
                onClick = function()
                    self:clear()
                    self.popupMenu:closeWindow()
                    self:onClose()
                    self:toggleHighliting(false)
                end
            }
        )
    end,

    toggleHighliting = function(self, on)
        if self.inventory.highlighting then
            if on and self.highlightingID == nil then 
                check = function() 
                    if self.inventory:compareCurrent(1) == 1 or self.inventory:compareCurrent(2) == 1 or 
                    self.inventory:compareCurrent(1) == 1 or self.inventory:spellbookCompare() == 1 then 
                        self:clear()
                        self:initialize(self.inventory)
                    end
                end
                self.highlightingID = osrs.Events:subscribe(function() check() end, osrs.Events.ON_GAME_TICK)
            elseif on == false and self.highlightingID ~= nil then
                osrs.Events:unsubscribe(self.highlightingID)
                self.highlightingID = nil
            end
        end
    end,

    onSwap = function(self, book)
        self.inventory:swapSpellbook(book)
        self:clear()
        self:initialize(self.inventory)
    end,

    onClick = function(self, item, index, table)
        nameTable = {
            [1] = "Inventory",
            [2] = "Equipment",
            [3] = "Rune Pouch",
            [4] = "Bolt Pouch",
        }
        self.popupMenu.window.title = nameTable[table]
        local updateSlotButton = osrs.Ui:textButton(
            {
                text = string.format("Update Slot from %s", nameTable[table]),
                onClick = function()
                    self.popupMenu:closeWindow()
                    self.inventory:copySlot(index, table)
                    self:clear()
                    self:initialize(self.inventory)
                end
            }
        )
        local updateAllSlotButton = osrs.Ui:textButton(
            {
                text = string.format("Update ALL slots from %s", nameTable[table]),
                onClick = function()
                    self.popupMenu:conformationPopup("Update All Setups", 
                    "Do you want to update ALL setups which have this item to the new item?",
                    false,
                    "",
                    function()
                        self.onChange(item, index, table, self.inventory)
                    end)
                end
            }
        )
        local stackHeader = osrs.Ui:collapsingHeader(
            {
                text = "Stack Indicator",
            }
        )
        if table == 1 or table == 2 then 
            local noneButton = osrs.Ui:textButton(
                {
                    text = "Stack Diffrence None",
                    onClick = function()
                        self.popupMenu:closeWindow()
                        item.stackDiffrence = nil
                        self:clear()
                        self:initialize(self.inventory)
                    end
                }
            )
            local standardButton = osrs.Ui:textButton(
                {
                    text = "Stack Diffrence Standard",
                    onClick = function()
                        self.popupMenu:closeWindow()
                        item.stackDiffrence = 1
                        self:clear()
                        self:initialize(self.inventory)
                    end
                }
            )
            local lessButton = osrs.Ui:textButton(
                {
                    text = "Stack Diffrence Less Than",
                    onClick = function()
                        self.popupMenu:closeWindow()
                        item.stackDiffrence = 2
                        self:clear()
                        self:initialize(self.inventory)
                    end
                }
            )
            local greaterButton = osrs.Ui:textButton(
                {
                    text = "Stack Diffrence Greater Than",
                    onClick = function()
                        self.popupMenu:closeWindow()
                        item.stackDiffrence = 3
                        self:clear()
                        self:initialize(self.inventory)
                    end
                }
            )
            stackHeader:addChild(noneButton)
            stackHeader:addChild(standardButton)
            stackHeader:addChild(lessButton)
            stackHeader:addChild(greaterButton)
        end
        local deleteButton = osrs.Ui:textButton(
            {
                text = "Remove Item",
                onClick = function()
                    self.popupMenu:closeWindow()
                    self.inventory:removeItem(index, table)
                    self:clear()
                    self:initialize(self.inventory)
                end
            }
        )
        local ui = {updateSlotButton, updateAllSlotButton, deleteButton}
        if table == 1 or table == 2 then 
            ui = {updateSlotButton, updateAllSlotButton, deleteButton, stackHeader}
        end
        self.popupMenu:showWindow(ui)
    end,

    show = function(self)
        self.panel:addChild(self.resetButton)
        self.invPanel:show(self.panel)
        if self.inventory:hasItem("rune pouch") == 1 then
            self.runePanel:show(self.panel)
        end
        self.equipPanel:show(self.panel)
        self.spellbookPanel:show(self.panel)
        self.panel:addChild(self.notesTitle)
        self.panel:addChild(self.notesText)
        self.panel:addChild(self.closeButton)
        self.window:addChild(self.panel)
        ThemeTool:update(self.window)
        osrs.Ui.canvas:addChild(self.window)
    end,

    clear = function(self)
        self.inventory.notes = self.notesText.value
        self.panel:removeChild(self.resetButton)
        self.invPanel:clear(self.panel)
        if self.inventory:hasItem("rune pouch") == 1 then
            self.runePanel:clear(self.panel)
        end
        self.equipPanel:clear(self.panel)
        self.spellbookPanel:clear(self.panel)
        self.panel:removeChild(self.notesTitle)
        self.panel:removeChild(self.notesText)
        self.panel:removeChild(self.closeButton)
        self.window:removeChild(self.panel)
        osrs.Ui.canvas:removeChild(self.window)
    end

}

return InventorySetWindow