require("./item")

-- local tables --
-- fix for the time being, find a way around this later
local runeTable = {
    [1] = 556,
    [2] = 555,
    [3] = 557, 
    [4] = 554,
    [5] = 558,
    [6] = 562,
    [7] = 560,
    [8] = 565,
    [9] = 564,
    [10] = 561,
    [11] = 563,
    [12] = 559,
    [13] = 566,
}

local equipmentTable = {
    [1] = osrs.ClientOp.WornSlots.HEAD,
    [2] = osrs.ClientOp.WornSlots.CAPE,
    [3] = osrs.ClientOp.WornSlots.AMULET,
    [4] = osrs.ClientOp.WornSlots.AMMO,
    [5] = osrs.ClientOp.WornSlots.WEAPON,
    [6] = osrs.ClientOp.WornSlots.BODY,
    [7] = osrs.ClientOp.WornSlots.SHIELD,
    [8] = osrs.ClientOp.WornSlots.LEGS,
    [9] = osrs.ClientOp.WornSlots.GLOVES, 
    [10] = osrs.ClientOp.WornSlots.BOOTS,
    [11] = osrs.ClientOp.WornSlots.RING
}
local spellbookTypes = {}
spellbookTypes["id"] = {780, 1583, 1584, 1711, -1}
spellbookTypes["name"] = {"Standard", "Ancient", "Lunar", "Arceuus", ""}

Inventory = {
    name = "nil",
    invItems = nil,
    equipment = nil,
    runePouch = nil,
    spellbook = nil,
    boltPouch = nil,
    notes = "",
    sections = nil,
    favorite = nil,
    ordered = nil,
    highlighting = nil,
    index = nil,
    backpack = 93,
    worn = 94,
    runeSlotVarbit = {29, 1622, 1623},
    runeAmountVarbit = {1624, 1625, 1626},
}

function Inventory:new(o, name)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    o:reset(name)
    o:setupInv()
    o:setupEquip()
    o:setupRunePouch()
    o:setupSpellbook()
    return o
end

function Inventory:remake(o, name, set)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    o:reset(name)
    o:setupAll(set)
    return o
end

function Inventory:reset(name)
    self.name = name
    self.invItems = {}
    self.equipment = {}
    self.runePouch = {}
    self.spellbook = nil
    self.boltPouch = {}
    self.notes = ""
    self.sections = {}
    self.favorite = false
    self.ordered = false
    self.highlighting = false
    self.index = 0
end

function Inventory:setupInv()
    local invSize = osrs.ClientOp.inv_size(self.backpack)
    for i = 0, invSize - 1, 1 do
        self.invItems[i + 1] = Item:new(
            nil, 
            osrs.ClientOp.inv_getobj(self.backpack, i), 
            osrs.ClientOp.inv_getnum(self.backpack, i))
    end
end

function Inventory:setupEquip()
    for index, slot in pairs(equipmentTable) do
        self.equipment[index] = Item:new(
            nil, 
            osrs.ClientOp.inv_getobj(self.worn, slot), 
            osrs.ClientOp.inv_getnum(self.worn, slot))
    end
end

function Inventory:setupRunePouch()
    for index, slot in pairs(self.runeSlotVarbit) do
        item = runeTable[osrs.ClientState:getVarbit(slot)]
        if item == nil or item == 0 then 
            self.runePouch[index] = Item:new(nil, -1, 0)
        else
            amount = osrs.ClientState:getVarbit(self.runeAmountVarbit[index])
            self.runePouch[index] = Item:new(nil, item, amount)
        end
    end
end

function Inventory:setupSpellbook()
    item = spellbookTypes["id"][osrs.ClientState:getVarbit(4070) + 1]
    self.spellbook = Item:new(nil, item, 0)
    self.spellbook.name = spellbookTypes["name"][osrs.ClientState:getVarbit(4070) + 1]
end

function Inventory:getTable(num)
    tables = {
        [1] = self.invItems,
        [2] = self.equipment,
        [3] = self.runePouch,
        [4] = self.boltPouch,
    }
    return tables[num]
end

function Inventory:setupAll(set)
    self.favorite = set["other"][1]
    self.ordered = set["other"][2]
    self.highlighting = set["other"][3]
    self.notes = set["other"][4]
    for index, section in pairs(set["sections"]) do
        table.insert(self.sections, section)
    end
    for index, item in pairs(set["items"]) do
        if index < 29 then 
            self.invItems[index] = Item:remake(nil, item)
        elseif index < 32 then
            self.runePouch[index - 28] = Item:remake(nil, item)
        elseif index < 43 then 
            self.equipment[index - 31] = Item:remake(nil, item)
        else
            self.spellbook = Item:remake(nil, item)
            for index, sb in pairs(spellbookTypes["id"]) do
                if sb == item.id then
                    self.spellbook.name = spellbookTypes["name"][index]
                end
            end
        end
    end
end

function Inventory:removeItem(index, group)
    group = self:getTable(group)
    group[index] = Item:new(nil, -1, 0)
end

function Inventory:addSections(list)
    for index, section in pairs(list) do
        found = 0
        for i, s in pairs(self.sections) do
            if section == s then
                found = 1
            end
        end
        if found == 0 then
            table.insert(self.sections, section)
        end
    end
end

function Inventory:compareCurrent(group)
    invGroup = self:getTable(group)
    current = Inventory:new(nil, current)
    currentGroup = current:getTable(group)
    if self.ordered then 
        return self:orderedCompare(invGroup, currentGroup)
    else
        return self:unorderedCompare(invGroup, currentGroup)
    end
end
-- rename these, they are flipped
function Inventory:orderedCompare(invGroup, currentGroup)
    change = 0
    for index, invItem in pairs(invGroup) do
        found = 0
        for i, currentItem in pairs(currentGroup) do
            if invItem.name == currentItem.name and found == 0 then 
                if invItem.highlited ~= false then
                    invItem.highlited = false
                    change = 1
                end
                found = 1
                currentItem.name = ""
            end
        end
        if invItem.highlited == false and found == 0 then 
            invItem.highlited = true
            change = 1
        end
    end
    return change
end

function Inventory:unorderedCompare(invGroup, currentGroup)
    change = 0
    for index, invItem in pairs(invGroup) do
        if (invItem.name ~= currentGroup[index].name or invItem.stackDiffrence ~= nil) and invItem.highlited == false then
            if invItem.stackDiffrence == nil or not invItem.stackTypes[invItem.stackDiffrence](currentGroup[index]) then
                invItem.highlited = true
                change = 1
            end
        elseif invItem.name == currentGroup[index].name and invItem.highlited then
            if invItem.stackDiffrence == nil or invItem.stackTypes[invItem.stackDiffrence](currentGroup[index]) then
                invItem.highlited = false
                change = 1
            end
        end
    end
    return change
end

function Inventory:spellbookCompare()
    change = 0
    current = Inventory:new(nil, current)
    if current.spellbook.id == self.spellbook.id and self.spellbook.highlited ~= false then
        self.spellbook.highlited = false
        change = 1
    elseif current.spellbook.id ~= self.spellbook.id and self.spellbook.highlited ~= true then
        self.spellbook.highlited = true
        change = 1
    end
    return change
end

function Inventory:alphabeticalCompare(a, b)
    return string.lower(a.name) < string.lower(b.name)
end

function Inventory:swapSpellbook(book)
    item = spellbookTypes["id"][book]
    self.spellbook = Item:new(nil, item, 0)
    self.spellbook.name = spellbookTypes["name"][book]
end

function Inventory:copySlot(index, group)
    if group == 1 then 
        self:copyInvSlot(index)
    elseif group == 2 then 
        self:copyEquipSlot(index)
    elseif group == 3 then 
        self:copyRuneSlot(index)
    end
end

function Inventory:changeItem(oldItem, newItemIndex, group)
    invGroup = self:getTable(group)
    current = Inventory:new(nil, current)
    currentGroup = current:getTable(group)
    newItem = currentGroup[newItemIndex]
    if newItem.id == -1 or oldItem == -1 then 
        return -1
    end
    for index, item in pairs(invGroup) do
        if oldItem.name == item.name then
            invGroup[index] = Item:new(nil, newItem.id, newItem.quantity)
        end
    end
    return 0
end

function Inventory:hasItem(name)
    if self:checkForItem(name, self.invItems) == 1 
    or self:checkForItem(name, self.equipment) == 1 
    or self:checkForItem(name, self.runePouch) == 1 
    or self:checkForSpellbook(name) == 1 then 
        return 1
    end
    return 0
end

function Inventory:checkForSpellbook(name)
    if string.lower(string.sub(self.spellbook.name, 1, #name)) == name then
        return 1
    end
    return 0
end

function Inventory:checkForItem(name, list)
    for index, item in pairs(list) do
        if self:checkWords(name, item.name) == 1 then 
            return 1
        end
    end
    return 0
end

function Inventory:checkWords(words, item)
    local count = 0
    local givenWords = self:splitName(words)
    local otherWords = self:splitName(item)
    for index, given in pairs(givenWords) do
        for i, word in pairs(otherWords) do
            if string.lower(given) == string.lower(string.sub(word, 1, #given)) then 
                count = count + 1
            end
        end
    end
    if count == #givenWords then 
        return 1
    end
    return 0
end

function Inventory:splitName(str)
    local words = {}
    for word in string.gmatch(str, "%S+") do
        table.insert(words, word)
    end
    return words
end

function Inventory:copyInvSlot(index)
    self.invItems[index] = Item:new(
        nil, 
        osrs.ClientOp.inv_getobj(self.backpack, index - 1), 
        osrs.ClientOp.inv_getnum(self.backpack, index - 1))
end

function Inventory:copyEquipSlot(index)
    self.equipment[index] = Item:new(
        nil, 
        osrs.ClientOp.inv_getobj(self.worn, equipmentTable[index]),
        osrs.ClientOp.inv_getnum(self.worn, equipmentTable[index]))
end

function Inventory:copyRuneSlot(index)
    item = runeTable[osrs.ClientState:getVarbit(self.runeSlotVarbit[index])]
    if item == nil then 
        self.runePouch[index] = Item:new(nil, 0, 0)
    else
        amount = osrs.ClientState:getVarbit(self.runeAmountVarbit[index])
        self.runePouch[index] = Item:new(nil, item, amount)
    end
end