local InventorySetupWindow = require("UI/inventorySetupWindow")
local InventorySetWindow = require("UI/inventorySetWindow")
local InventoryWindowPopup = require("UI/inventoryWindowPopup")
local InventoryInfo = require("inventoryInfo")
local ThemeTool = require("UI/themeTool")
require("asyncTaskHandler")

local InventoryController = {
    startup = function(self)
        if InventoryWindowPopup.onCreate == nil then 
            InventoryWindowPopup.onCreate = function(item, isSet) self.create(item, isSet) end
            InventoryWindowPopup.onDelete = function(name, isSet) self.delete(name, isSet) end
            InventoryWindowPopup.onRename = function(oldName, newName, isSet) self.rename(oldName, newName, isSet) end
            InventoryWindowPopup.onMove = function(item, oldIndex, newIndex) self.move(item, oldIndex, newIndex) end
            InventorySetupWindow.onClose = function() self.saveAll() end
            InventorySetupWindow.onRefresh = function() self.refresh() end
            InventoryWindowPopup.onRefresh = function() self.refresh() end
            InventorySetupWindow.onExport = function(item) self.export(item) end
            InventoryWindowPopup.onImport = function(info, isSet) self.import(info, isSet) end
            InventorySetWindow.onRefresh = function(inv) self.resetInv(inv) end
            InventorySetWindow.onClose = function() self.openHomeWindow() end
            InventorySetWindow.onChange = function(item, index, group, inv) self.changeItem(item, index, group, inv) end
            InventoryInfo.displayErrorMessage = function(message) InventoryWindowPopup:errorPopup(message) end
            InventorySetupWindow.popupMenu = InventoryWindowPopup
            InventorySetWindow.popupMenu = InventoryWindowPopup
        end
        local taskList = AsyncTaskList.New() 
        AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:initialize(callback) end)  
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end)
        InventoryWindowPopup:initialize()
        ThemeTool:initialize()
        AsyncTaskList.runTaskList(taskList)
    end,

    create = function(item, isSet)
        local taskList = AsyncTaskList.New() 
        if isSet then
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:createInventory(item, callback) end)  
        else 
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:createSection(item, callback) end)  
        end
        -- not sure how to call this in another method
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:clear() callback() end)
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end)
        AsyncTaskList.runTaskList(taskList)
    end,

    delete = function(name, isSet)
        local taskList = AsyncTaskList.New() 
        if isSet then
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:deleteInventory(name) callback() end)  
        else
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:deleteSection(name) callback() end)  
        end
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:clear() callback() end)
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end)
        AsyncTaskList.runTaskList(taskList)
    end,

    rename = function(oldName, newName, isSet)
        local taskList = AsyncTaskList.New() 
        if isSet then
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:renameSet(oldName, newName, isSet) callback() end)  
        else
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:renameSection(oldName, newName, isSet) callback() end) 
        end
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:clear() callback() end)
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end)
        AsyncTaskList.runTaskList(taskList)
    end,

    move = function(item, oldIndex, newIndex)
        local taskList = AsyncTaskList.New() 
        if InventorySetupWindow.sortingMode then
            AsyncTaskList.schedule(taskList,function(callback) InventoryWindowPopup:errorPopup("You cannot move this while in sorting mode") callback() end)
        else
            if item.index ~= nil then
                AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:moveItem(item, oldIndex, newIndex, InventoryInfo.savedSets, callback) end)  
            else
                AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:moveItem(item, oldIndex, newIndex, InventoryInfo.savedSections, callback) end)  
            end
            AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:clear() callback() end)
            AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end)
        end
        AsyncTaskList.runTaskList(taskList)
    end,

    saveAll = function()
        InventorySetupWindow:clear()
        InventoryInfo:saveSets()
        ThemeTool:onClose()
    end,

    refresh = function()
        local taskList = AsyncTaskList.New() 
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:clear() callback() end)
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end)
        AsyncTaskList.runTaskList(taskList)
    end,

    export = function(item)
        local taskList = AsyncTaskList.New() 
        local str = ""
        if (type(item) ~= "string") then 
            AsyncTaskList.schedule(taskList,function(callback) str = InventoryInfo:writeInventory(item) callback() end)
        else
            AsyncTaskList.schedule(taskList,function(callback) str = InventoryInfo:writeSection(item) callback() end)
        end
        AsyncTaskList.schedule(taskList,function(callback) InventoryWindowPopup:conformationPopup(
            "Exporting",
            "Copy from the textbox bellow",
            true,
            str,
            function() end) callback() 
        end)
        AsyncTaskList.runTaskList(taskList)
    end,

    import = function(info, isSet)
        local taskList = AsyncTaskList.New() 
        if isSet then
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:importSetup(info, callback) end)
        else
            AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:importSection(info, callback) end)
        end
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:clear() callback() end)  
        AsyncTaskList.schedule(taskList,function(callback) InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections) callback() end) 
        AsyncTaskList.runTaskList(taskList)  
    end,

    resetInv = function(inv)
        local taskList = AsyncTaskList.New() 
        AsyncTaskList.schedule(taskList,function(callback) InventoryInfo:resetInventory(inv) callback() end) 
        AsyncTaskList.schedule(taskList,function(callback) InventorySetWindow:clear() callback() end)  
        AsyncTaskList.schedule(taskList,function(callback) InventorySetWindow:initialize(inv) callback() end)  
        AsyncTaskList.runTaskList(taskList)  
    end,

    openHomeWindow = function()
        InventorySetupWindow:initialize(InventoryInfo.savedSets, InventoryInfo.savedSections)
        InventorySetupWindow:checkForSearch(true, InventoryInfo.savedSets, InventoryInfo.savedSections)
    end,

    changeItem = function(item, index, group, inv)
        local taskList = AsyncTaskList.New()
        changedItems = 0
        AsyncTaskList.schedule(taskList,function(callback) changedItems = InventoryInfo:changeItem(item, index, group) callback() end) 
        AsyncTaskList.schedule(taskList,function(callback) InventorySetWindow:clear() callback() end)  
        AsyncTaskList.schedule(taskList,function(callback) InventorySetWindow:initialize(inv) callback() end)
        AsyncTaskList.schedule(taskList,function(callback) 
            if changedItems == -1 then 
                InventoryWindowPopup:errorPopup("You cannot update empty slots or replace all slots with this item with an empty slot")
            end callback() end) 
        AsyncTaskList.runTaskList(taskList) 
    end,
}

return InventoryController